/*
 * Copyright (c) 2004 Teodor Sigaev <teodor@sigaev.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *        may be used to endorse or promote products derived from this software
 *        without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <string.h>

#include "connection.h"
#include "tlog.h"
#include "tmalloc.h"


void
TC_addConnection(PoolConnection *pool, TC_Connection *c) {
	if (!c)
		return;
	if ( pool->number >= pool->len ) {
		if ( pool->len == 0 ) {
			pool->len = 8;
			pool->conn = (TC_Connection**)tmalloc( sizeof(TC_Connection*) * pool->len );
		} else {
			pool->len *= 2;
			pool->conn = (TC_Connection**)trealloc( pool->conn, sizeof(TC_Connection*) * pool->len );
		}
	} 
	pool->conn[ pool->number ] = c;
	pool->number++;
}

void
TC_deleteConnectionByN(PoolConnection *pool, int n) {
	if ( n<0 || n>=pool->number || pool->len<=0 ) 
		return;

	TC_FreeConnection( pool->conn[ n ] );
	free( pool->conn[ n ] );
	if ( n==pool->number-1 ) {
		pool->number--;
		return;
	} 

	memcpy( &( pool->conn[ n ] ), &( pool->conn[ n+1 ] ), sizeof(TC_Connection*) * ( pool->number - n - 1 )); 
	pool->number--;
	return;
}

void
TC_deleteConnectionByC(PoolConnection *pool, TC_Connection *c) {
	int i;
	for(i=0;i<pool->number;i++)
		if ( pool->conn[ i ] == c ) {
			TC_deleteConnectionByN(pool, i);
			break;
		}
}
