#include <errno.h>
#include <stdlib.h>   
#include <string.h>
         
#include "tlog.h"
#include "tmalloc.h"

#include "tsearchd.h"

#include "postgres.h"
#include "query.h"
#include "rewrite.h"

/*
static void
showtree(TDNode *node, u_int32_t level) {
        if ( !node ) {
                tlog(TL_DEBUG, "%d NODE IS NULL", level);
                return;
        }

	switch(node->type) {
		case TD_VAL:	
               		tlog(TL_DEBUG, "%d V: '%s' f:%c c:%c",
				level,
                       		node->word,
                       		(node->isfinita) ? 'Y' : 'N',
                       		(node->cancache) ? 'Y' : 'N'
               		);
			break;
		case TD_AND:
			tlog(TL_DEBUG, "%d & f:%c", level, (node->isfinita) ? 'Y' : 'N');
			break;
		case TD_OR:
			tlog(TL_DEBUG, "%d | f:%c", level, (node->isfinita) ? 'Y' : 'N');
			break;
		case TD_NOT:
			tlog(TL_DEBUG, "%d ! f:%c", level, (node->isfinita) ? 'Y' : 'N');
			break;
		case TD_MINES:
			tlog(TL_DEBUG, "%d - f:%c", level, (node->isfinita) ? 'Y' : 'N');
			break;
		default:
			tlog(TL_DEBUG, "%d @@@@@@@@@@@@@@ f:%c", level, (node->isfinita) ? 'Y' : 'N');
	} 

	if ( node->left )
		showtree(node->left, level+1);
	if ( node->right )
		showtree(node->right, level+1);
}
*/

/*
 * make query tree from tsquery
 */
static TDNode *
makeTree(QUERYTYPE *tsquery, ITEM * in)
{
        TDNode       *node = (TDNode *) t0malloc(sizeof(TDNode));

        if (in->type == OPR)
        {
                node->left = makeTree(tsquery, in + 1);
                if (in->val != (int4) '!')
                        node->right = makeTree(tsquery, in + in->left);
		switch( in->val ) {
			case '!':   node->type = TD_NOT; break;
			case '|':   node->type = TD_OR;  break;
			case '&':   node->type = TD_AND; break;
			default:
				tlog(TL_CRIT|TL_EXIT,"makeTree: Unknown type of operation: %c", in->val);
		}
        } else {
		node->type = TD_VAL;
		node->word = tmalloc( in->length + 1 );
		memcpy( node->word, GETOPERAND(tsquery) + in->distance, in->length );
		node->word[ in->length ] = '\0'; 
	}
        return node;
}

static void
freeTree(TDNode *node) {
	if ( !node )
		return;
	if ( node->left )
		freeTree( node->left );
	if ( node->right )
		freeTree( node->right );
	if ( node->word );
		tfree( node->word );
	tfree(node);
}

void
freePolish(TDNode *node) {
	TDNode *ptr=node;
	while(node) {
		if ( node->word )
			tfree( node->word );
		ptr=node->next;
		tfree(node);
		node = ptr;
	}
}

static TDNode *
replaceNot(TDNode *node) {
	if ( node->type == TD_AND ) {
		node->left = replaceNot( node->left );
		node->right = replaceNot( node->right );
		if ( node->left->type == TD_NOT && node->right->type == TD_NOT ) {
			node->left->right = node->right->left;
			node->right->left=NULL;
			freeTree( node->right );
			node->left->type=TD_OR;
			node->type = TD_NOT;
			node->right = NULL;
		} else if ( node->left->type == TD_NOT ) {
			TDNode *res = node->left->left;
			node->left->left = NULL;
			freeTree( node->left );
			node->left = node->right;
			node->right = res;
			node->type = TD_MINES;
		} else if ( node->right->type == TD_NOT ) {
			TDNode *res = node->right->left;
			node->right->left=NULL;
			freeTree( node->right );
			node->right = res;
			node->type = TD_MINES;
		}
	} else if ( node->type == TD_OR ) {
		node->left = replaceNot( node->left );
		node->right = replaceNot( node->right );
		if ( node->left->type == TD_NOT && node->right->type == TD_NOT ) {
			node->left->right = node->right->left;
			node->right->left=NULL;
			freeTree( node->right );
			node->left->type=TD_AND;
			node->type = TD_NOT;
			node->right = NULL;
		} else if ( node->right->type == TD_NOT ) {
			TDNode       *res = node->right;
			node->right = res->left;
			res->left = node;
			node->type=TD_MINES;
			node = res;
		} else if ( node->left->type == TD_NOT ) {
			TDNode       *res = node->left;
			node->left = node->right;
			node->right = res->left;
			res->left = node;
			node->type=TD_MINES;
			node = res;
		}
	} else if ( node->type == TD_NOT ) {
		node->left = replaceNot( node->left );

		if ( node->left->type == TD_NOT ) {
			TDNode *res = node->left->left;
			node->left->left = NULL;
			freeTree(node);
			node=res;
		}
	}
	return node;
}

static TDNode *
makePolish(TDNode *root) {
	TDNode *res = root;
	
	if ( root->type != TD_VAL ) {
		TDNode *ptr;

		res=ptr=makePolish( root->left );
		while( ptr->next )
			ptr=ptr->next;

		ptr->next = makePolish( root->right );
		while( ptr->next )
			ptr=ptr->next;
		
		ptr->next = root;
	}
	
	return res;	
}


int
prepareQuery(QUERYTYPE *tsquery, TDNode** res) {

	*res = makeTree(tsquery, GETQUERY(tsquery));
	if ( !*res ) 
		return TSD_VOIDQUERY;
	
	*res = replaceNot( *res );
	if ( !*res ) 
		return TSD_VOIDQUERY;
	if ( (*res)->type == TD_NOT ) 
		return TSD_NOTQUERY;

	*res = makePolish( *res );

	return TSD_OK;
} 

