#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <sys/sysctl.h>

#include "tlog.h"
#include "connection.h"
#include "tmalloc.h"

#include "tsearchd.h"

static int continueWork=1;

static void
inquireExit(int s) {
        continueWork=0;
}


extern char *optarg;
extern int optind;
int Debug=FALSE;

static void
usage() {
	fputs(
		"Usage:\n"
		"tsearchd [-D] [-a ADDRESS] [-p PORT] [-m MEMCACHE_MB] -d DIRECTORY\n"
	, stdout);
	exit(1);
}

int
main( int argc, char *argv[] ) {
	int ch;
	int child=0, listenport = TSEARCHD_PORT;
	char *host="127.0.0.1";
	char *directory=NULL;
	char buf[1024];
	u_int32_t memcache = 0;

	while( (ch=getopt(argc, argv, "hp:a:d:Dm:"))!=-1) {
		switch(ch) {
			case 'D':
				Debug=TRUE;
				break;
			case 'p':
				listenport=atoi(optarg);
				break;
			case 'm':
				memcache=(u_int32_t)atoi(optarg);
				break;
			case 'd':
				directory=tstrdup(optarg);
				break;
			case 'a':
				host=tstrdup(optarg);
				break;
			case 'h':
			default:
				usage();
		}
	}

	if ( !directory ) usage();

	if ( Debug )
		opentlog( TL_OPEN_STDERR,  TL_DEBUG, NULL);
	else
		opentlog( TL_OPEN_SYSLOG, TL_INFO, NULL);

	if  ( Debug || (child = fork()) == 0 ) {
		/* child */
		if  ( Debug || (child = fork()) == 0 ) {
			/*child*/
			TC_Connection *newconn;
			PoolConnection	pool;
			TDStorage *storage;
			int i;

			signal(SIGINT, inquireExit);
			signal(SIGHUP, inquireExit);

			sprintf(buf,"%s/tsearchd.idx", directory);
			storage = TDOpenStorage( buf );
			if (!storage)
				tlog(TL_CRIT|TL_EXIT,"Can't open storage at %s\n", buf);
			if (memcache)
				storage->maxallocated = memcache*1024*1024;

			memset(&pool, 0, sizeof(PoolConnection));
			TC_addConnection( &pool, TC_fillConnection(NULL, host, listenport) );
			TC_ClientInitConnection( pool.conn[0], host, listenport);
			pool.conn[0]->state = CS_READ;

			while(continueWork) {
				if ( TC_ReadyIO(pool.conn, pool.number, -1) ) {
					if ( pool.conn[0]->readyio && (newconn=TC_AcceptTcp( pool.conn[0] )) != NULL ) {
						if ( newconn->state == CS_CONNECTED ) {
							newconn->state = CS_READ;
							TC_addConnection( &pool, newconn );
							tlog(TL_DEBUG,"Accept new connection... total: %d", pool.number);
						}
					}
					ConnectionWorks(&pool, storage);
				}
			}
			for(i=0;i<pool.number; i++)
				TC_deleteConnectionByN(&pool,i);

			TDCloseStorage(storage);
					
		}
	}

	if (child == -1)
		tlog(TL_CRIT|TL_EXIT,"Can't start child process");

	return 0;
}

