#include <errno.h>
#include <stdlib.h>
#include <string.h>
                 
#include "tlog.h"
#include "tmalloc.h"

#include "tsearchd.h"

static void
msgFlush( TC_Connection *conn, TDStorage *storage ) {
	TCMsg *pmsg;
	TDFlushStorage(storage);
	conn->len = TCMSGHDRSZ + sizeof(u_int32_t);
	conn->buf = trealloc( conn->buf, conn->len );

	pmsg = (TCMsg*) conn->buf;
	pmsg->len = conn->len;
	pmsg->type = TSM_FLUSH;
	*(u_int32_t*)(pmsg->data) = TSD_OK;
	TC_Send(conn);
}

static char bufWordPut[ MAXSTRLEN + 1 ] ; /* WordEntry:len in tsvector.h */

static void
msgPut( TC_Connection *conn, TDStorage *storage, u_int32_t type ) {
	TCMsg *pmsg = (TCMsg*) conn->buf;
	TSIndexData	*toadd = (TSIndexData*) pmsg->data;
	int32_t i;
	
	char *ptr = STRPTR((toadd->data));
	WordEntry *we = ARRPTR((toadd->data));

	for(i=0; i<toadd->data->size; i++) {
		memcpy( bufWordPut, ptr + we[i].pos, we[i].len );
		bufWordPut[ we[i].len ] = '\0';
		TDAppend(storage, bufWordPut, toadd->id); 
	}

	if ( storage->allocated >= storage->maxallocated ) 
		TDFlushStorage(storage);

	conn->len = TCMSGHDRSZ + sizeof(u_int32_t);
	conn->buf = trealloc( conn->buf, conn->len );

	pmsg = (TCMsg*) conn->buf;
	pmsg->len = conn->len;
	pmsg->type = type;
	*(u_int32_t*)(pmsg->data) = TSD_OK;

	TC_Send(conn);	
}

static WordData tmpWordData;

static void
msgSearch( TC_Connection *conn, TDStorage *storage ) {
	TCMsg *pmsg = (TCMsg*) conn->buf;
	QUERYTYPE  *tsquery = (QUERYTYPE*) pmsg->data;
	TDNode *query;
	WordData *result = NULL;
	
	int rc;

	if ( (rc=prepareQuery(tsquery, &query))!=TSD_OK ) {
		result = &tmpWordData;
		result->num=0;
		result->status = rc;
	} else {
		result = TDExecute(storage, query); 
		if ( !result ) {
			result = &tmpWordData;
			result->num=0;
		}
		result->status = TSD_OK; 
	}
	
	freePolish(query);
	conn->len = TCMSGHDRSZ + WDHRDRSZ + sizeof(int32_t) * result->num;
	conn->buf = trealloc( conn->buf, conn->len );

	pmsg = (TCMsg*) conn->buf;
	pmsg->len = conn->len;
	pmsg->type = TSM_QUERY;
	memcpy( pmsg->data, result, WDHRDRSZ + sizeof(int32_t) * result->num );
	if ( result != &tmpWordData )
		tfree(result);

	TC_Send(conn); 
}

static void
gotMessage( TC_Connection *conn, TDStorage *storage ) {
	TCMsg *pmsg = (TCMsg*) conn->buf;
	if ( conn->ptr - conn->buf < TCMSGHDRSZ ) {
		tlog(TL_ALARM,"Wrong size of long message (%d bytes)", conn->ptr - conn->buf);
		conn->state = CS_ERROR;
		return;
	}

	tlog(TL_DEBUG,"Got 0x%x message len %d bytes", pmsg->type, pmsg->len); 
	switch(pmsg->type) {
		case 	TSM_QUERY:   msgSearch(conn, storage); break;
		case 	TSM_BULKPUT:   
		case 	TSM_PUT:     msgPut   (conn, storage, pmsg->type); break;
		case 	TSM_FLUSH:   msgFlush (conn, storage); break;
		default:
			tlog(TL_ALARM,"Got unknown packet type: %d", pmsg->type);
			conn->state = CS_ERROR;
	}
}

static void
sendedMessage( TC_Connection *conn, TDStorage *storage ) {
	TCMsg *pmsg = (TCMsg*) conn->buf;
	if ( conn->ptr - conn->buf < TCMSGHDRSZ ) {
		tlog(TL_ALARM,"Wrong size of long message (%d bytes)", conn->ptr - conn->buf);
		conn->state = CS_ERROR;
		return;
	}

	tlog(TL_DEBUG,"Sended 0x%x message len %d bytes", pmsg->type, pmsg->len); 
	switch(pmsg->type) {
		case    TSM_BULKPUT: conn->state = CS_READ; conn->ptr=conn->buf; break;
		case	TSM_QUERY:
		case	TSM_PUT:
		case	TSM_FLUSH:
		default:
			conn->state = CS_FINISH;
	}
}

void
ConnectionWorks(PoolConnection *pool, TDStorage *storage) {
	int i;
	TC_Connection *conn;

	for(i=1; i<pool->number; i++) {
		conn = pool->conn[i];
		switch( conn->state ) {
			case CS_INPROCESS:
				TC_ServerConnect( conn, 0 );
				break;
			case CS_CONNECTED: /* we have connected to remote host */
				if ( conn->readyio ) {
					TC_Read( conn );
					if ( conn->state != CS_READ ) i--; /* check it one more time */
				}
				break;
			case CS_FINISHSEND:	
				sendedMessage( conn, storage );
				i--;
				break;
			case CS_FINISHREAD:
				gotMessage( conn, storage );
				if (  conn->state != CS_FINISHREAD ) i--;
				break;
			case CS_READ:
				if ( conn->readyio ) {
					TC_Read( conn );
					if ( conn->state != CS_READ ) i--; /* check it one more time */
				}
				break;
			case CS_SEND:
				if ( conn->readyio ) {
					TC_Send( conn );
					if ( conn->state != CS_SEND ) i--; /* check it one more time */
				}
				break;
			/* ignore */
			case CS_OK:
			case CS_AGAIN:
			case CS_WAIT:
				break;
			case CS_TIMEOUT:
			case CS_CLOSED:
			case CS_NOTINITED:
			case CS_FINISH:
			case CS_ERROR:
			default:
				tlog(TL_DEBUG, "Clear N%d connection", i);
				TC_deleteConnectionByN(pool,i);
				i--;
		}
	}
}

